from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_o_auth_applications_response_200 import ListOAuthApplicationsResponse200
from ...models.list_o_auth_applications_response_400 import ListOAuthApplicationsResponse400
from ...models.list_o_auth_applications_response_403 import ListOAuthApplicationsResponse403
from ...models.list_o_auth_applications_response_422 import ListOAuthApplicationsResponse422
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    limit: Union[Unset, float] = 10.0,
    offset: Union[Unset, float] = 0.0,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/oauth_applications",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ListOAuthApplicationsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ListOAuthApplicationsResponse400.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ListOAuthApplicationsResponse403.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = ListOAuthApplicationsResponse422.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, float] = 10.0,
    offset: Union[Unset, float] = 0.0,
) -> Response[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    """Get a list of OAuth applications for an instance

     This request returns the list of OAuth applications for an instance.
    Results can be paginated using the optional `limit` and `offset` query parameters.
    The OAuth applications are ordered by descending creation date.
    Most recent OAuth applications will be returned first.

    Args:
        limit (Union[Unset, float]):  Default: 10.0.
        offset (Union[Unset, float]):  Default: 0.0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ListOAuthApplicationsResponse200, ListOAuthApplicationsResponse400, ListOAuthApplicationsResponse403, ListOAuthApplicationsResponse422]]
    """

    kwargs = _get_kwargs(
        limit=limit,
        offset=offset,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, float] = 10.0,
    offset: Union[Unset, float] = 0.0,
) -> Optional[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    """Get a list of OAuth applications for an instance

     This request returns the list of OAuth applications for an instance.
    Results can be paginated using the optional `limit` and `offset` query parameters.
    The OAuth applications are ordered by descending creation date.
    Most recent OAuth applications will be returned first.

    Args:
        limit (Union[Unset, float]):  Default: 10.0.
        offset (Union[Unset, float]):  Default: 0.0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ListOAuthApplicationsResponse200, ListOAuthApplicationsResponse400, ListOAuthApplicationsResponse403, ListOAuthApplicationsResponse422]
    """

    return sync_detailed(
        client=client,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, float] = 10.0,
    offset: Union[Unset, float] = 0.0,
) -> Response[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    """Get a list of OAuth applications for an instance

     This request returns the list of OAuth applications for an instance.
    Results can be paginated using the optional `limit` and `offset` query parameters.
    The OAuth applications are ordered by descending creation date.
    Most recent OAuth applications will be returned first.

    Args:
        limit (Union[Unset, float]):  Default: 10.0.
        offset (Union[Unset, float]):  Default: 0.0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ListOAuthApplicationsResponse200, ListOAuthApplicationsResponse400, ListOAuthApplicationsResponse403, ListOAuthApplicationsResponse422]]
    """

    kwargs = _get_kwargs(
        limit=limit,
        offset=offset,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, float] = 10.0,
    offset: Union[Unset, float] = 0.0,
) -> Optional[
    Union[
        ListOAuthApplicationsResponse200,
        ListOAuthApplicationsResponse400,
        ListOAuthApplicationsResponse403,
        ListOAuthApplicationsResponse422,
    ]
]:
    """Get a list of OAuth applications for an instance

     This request returns the list of OAuth applications for an instance.
    Results can be paginated using the optional `limit` and `offset` query parameters.
    The OAuth applications are ordered by descending creation date.
    Most recent OAuth applications will be returned first.

    Args:
        limit (Union[Unset, float]):  Default: 10.0.
        offset (Union[Unset, float]):  Default: 0.0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ListOAuthApplicationsResponse200, ListOAuthApplicationsResponse400, ListOAuthApplicationsResponse403, ListOAuthApplicationsResponse422]
    """

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            offset=offset,
        )
    ).parsed
