from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.create_redirect_url_body import CreateRedirectURLBody
from ...models.create_redirect_url_response_200 import CreateRedirectURLResponse200
from ...models.create_redirect_url_response_400 import CreateRedirectURLResponse400
from ...models.create_redirect_url_response_422 import CreateRedirectURLResponse422
from ...types import Response


def _get_kwargs(
    *,
    body: CreateRedirectURLBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/redirect_urls",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = CreateRedirectURLResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = CreateRedirectURLResponse400.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = CreateRedirectURLResponse422.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateRedirectURLBody,
) -> Response[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    """Create a redirect URL

     Create a redirect URL

    Args:
        body (CreateRedirectURLBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateRedirectURLBody,
) -> Optional[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    """Create a redirect URL

     Create a redirect URL

    Args:
        body (CreateRedirectURLBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateRedirectURLBody,
) -> Response[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    """Create a redirect URL

     Create a redirect URL

    Args:
        body (CreateRedirectURLBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateRedirectURLBody,
) -> Optional[Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]]:
    """Create a redirect URL

     Create a redirect URL

    Args:
        body (CreateRedirectURLBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreateRedirectURLResponse200, CreateRedirectURLResponse400, CreateRedirectURLResponse422]
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed
